Option Explicit On 
Option Strict On
Imports System.Web.Services
Imports System.Web.Services.Protocols
Imports System.Data.SqlClient
Imports System.Xml
Imports System.IO

Public Class Authenticate
   Inherits SoapHeader
   Public ServerName As String
   Public UserName As String
   Public Password As String
End Class

<WebService(Namespace:="http://oakleaf.ws/services/crosstabs/", _
   Description:="The ReportsWS XML Web service provides methods to return " + _
   "a variety of crosstab(ular) financial reports from the OakLeaf Consumer " + _
   "Electronics Orders, Invoices, and Products tables. The service uses " + _
   "document/literal format for SOAP request and response messages.")> _
Public Class ReportsWS
   Inherits System.Web.Services.WebService
   Private strXML As String
   Private strCrLf As String = Chr(13) + Chr(10)
   Private strTitle As String = "2001 Quarterly Orders by State"
   Private blnNullToZero As Boolean = True
   Private blnRowTotals As Boolean = True
   Private blnColTotals As Boolean = True

   'Variables for creating T-SQL queries
   Private strStartDate As String = "1/1/2001"
   Private datStartDate As DateTime = #1/1/2001#
   Private strStartTime As String = "00:00:00 AM"
   Private strInterval As String = "Quarter"
   Private strTitleInt As String = "Quarterly"
   Private strDataCols As String = "4"
   Private strQuery As String
   Private blnStates As Boolean = True

   Public hdrAuthent As Authenticate

#Region " Web Services Designer Generated Code "

    Public Sub New()
        MyBase.New()

        'This call is required by the Web Services Designer.
        InitializeComponent()

        'Add your own initialization code after the InitializeComponent() call

    End Sub

    'Required by the Web Services Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Web Services Designer
    'It can be modified using the Web Services Designer.  
    'Do not modify it using the code editor.
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        components = New System.ComponentModel.Container()
    End Sub

    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        'CODEGEN: This procedure is required by the Web Services Designer
        'Do not modify it using the code editor.
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

#End Region

   <WebMethod(Description:="The TimeSeriesReport method accepts an element-centric XML document that specifies " + _
      "the data source and layout of a time-series crosstab report for orders received by state or product category. The method returns an element-centric " + _
      "XML document with headers and formatted row and column values."), SoapHeader("hdrAuthent")> _
   Public Function TimeSeriesReport(ByVal RequestDoc As String) As String
      'Read the request document and verify its elements
      Dim srdRequest As StringReader = Nothing
      Dim xtrRequest As XmlTextReader = Nothing
      Dim strTemp As String
      Dim strSQL As String
      Dim strResponse As String

      Try
         Trace.Write("Begin read request document")
         srdRequest = New StringReader(RequestDoc)
         xtrRequest = New XmlTextReader(srdRequest)
         With xtrRequest
            '(strTemp reads elements not used at this point)
            .MoveToContent()  'Root
            .Read()           'Skip <timeseries>
            strTitle = .ReadElementString("title")
            .Read()           'Skip
            .Read()           'Skip
            strTemp = .ReadElementString("datatable")
            If .ReadElementString("rowsby") = "State" Then
               blnStates = True
            Else
               blnStates = False
            End If
            strStartDate = .ReadElementString("startdate")
            strStartTime = .ReadElementString("starttime")
            strInterval = .ReadElementString("interval")
            strDataCols = .ReadElementString("datacolumns")
            strTemp = .ReadElementString("count")
            strTemp = .ReadElementString("avgvalue")
            If .ReadElementString("rowtotals") = "Yes" Then
               blnRowTotals = True
            Else
               blnRowTotals = False
            End If
            If .ReadElementString("coltotals") = "Yes" Then
               blnColTotals = True
            Else
               blnColTotals = False
            End If
            If .ReadElementString("nulltozero") = "Yes" Then
               blnNullToZero = True
            Else
               blnNullToZero = False
            End If
         End With
      Catch
         'No error processing at this point
      Finally
         srdRequest.Close()
         xtrRequest.Close()
      End Try
      'Generate the T-SQL statement
      strSQL = WriteOrdersQuery()
      'Execute the query and return the report document
      strResponse = GenerateReport(strSQL)
      Return strResponse
   End Function

   '********************************************
   'T-SQL Query and Report Generation Procedures
   '********************************************

   Private Function WriteOrdersQuery() As String
      'Generate the T-SQL query from request document values
      Dim strSelect As String
      Dim strField As String
      Dim strHeader As String
      Dim strPeriod As String
      Dim strYear As String
      Dim datLastDate As DateTime
      Dim strFrom As String
      Dim strWhere As String
      Dim strGroup As String
      Dim strOrder As String
      Dim strSQL As String

      Trace.Write("Begin generate T-SQL query")
      datStartDate = CDate(strStartDate)
      datLastDate = datStartDate
      strFrom = "FROM Customers " + _
         "INNER JOIN Orders ON Customers.CustID = Orders.CustID " + _
         "INNER JOIN LineItems ON Orders.OrderID = LineItems.OrderID "

      strSelect = "SELECT TOP 100 PERCENT "
      strYear = "DATEPART(year, OrderDate)"
      strWhere = "WHERE (OrderDate BETWEEN '" + strStartDate + " " + strStartTime + "' AND '"
      strGroup = "GROUP BY "
      strOrder = "ORDER BY "
      If blnStates Then
         strField = "State"
         strSelect += strField + ", "
      Else
         strField = "SUBSTRING(SKU, 1, 4)"
         strSelect += strField + " AS Category, "
      End If
      strGroup += strField + ", "
      strOrder += strField + ", "

      Select Case strInterval
         Case "Quarter"
            strHeader = "DATENAME(year, OrderDate) + 'Q' + DATENAME(quarter, OrderDate)"
            strPeriod = "DATEPART(quarter, OrderDate)"
            datLastDate = datLastDate.AddMonths(3 * CInt(strDataCols))
         Case "Month"
            strHeader = "SUBSTRING(DATENAME(month, OrderDate), 1, 3) + ' ' + DATENAME(year, OrderDate)"
            strPeriod = "DATEPART(month, OrderDate)"
            datLastDate = datLastDate.AddMonths(CInt(strDataCols))
         Case "Week"
            strHeader = "DATENAME(year, OrderDate) + 'W' + DATENAME(week, OrderDate)"
            strPeriod = "DATEPART(week, OrderDate)"
            datLastDate = datLastDate.AddDays(7 * CInt(strDataCols))
         Case "Day"
            strHeader = "CAST(DATEPART(mm, OrderDate) AS varchar) + '/' + CAST(DATEPART(dd, OrderDate) AS varchar) + '/' + CAST(DATEPART(yy, OrderDate) AS varchar)"
            strPeriod = "DATEPART(dy, OrderDate)"
            datLastDate = datLastDate.AddDays(CInt(strDataCols))
      End Select
      datLastDate = datLastDate.AddSeconds(-1)
      strGroup += strPeriod + ", " + strYear + ", " + strHeader + " "
      strOrder += strPeriod + ", " + strYear + ", " + strHeader + " "
      strPeriod += " AS Period, " + strYear + " AS Year "
      strSelect += strHeader + " AS Header, COUNT(Orders.OrderID) AS Orders, ROUND(SUM(Quantity * UnitPrice), 0) AS Amount, " + strPeriod
      strWhere += Format(datLastDate, "M/d/yyyy hh:mm:ss tt") + "') AND LineItems.Deleted = 0 "

      strSQL = strSelect + strCrLf + strFrom + strCrLf + strWhere + strCrLf + strGroup + strCrLf + strOrder
      Return strSQL
   End Function

   Private Function GenerateReport(ByVal strSQL As String) As String
      'Generate and populate array, and create basic XML response document
      'SOAP headers supply ServerName, UserName, and Password values
      Dim astrCT(,) As String
      Dim intCol As Int32
      Dim intRow As Int32
      Dim intTotal As Int32
      Dim datPeriod As DateTime
      Dim cnnCT As New SqlConnection()
      Dim cmmCT As SqlCommand
      Dim sdrCT As SqlDataReader
      Dim strTemp As String
      Dim strAttrib As String
      Dim strXML As String 'Response document

      Trace.Write("Begin create array")
      datPeriod = datStartDate
      If blnStates Then
         ReDim astrCT(13, CInt(strDataCols) + 1)
         astrCT(0, 0) = "State"
         astrCT(1, 0) = "CO"
         astrCT(2, 0) = "IA"
         astrCT(3, 0) = "KS"
         astrCT(4, 0) = "MO"
         astrCT(5, 0) = "MT"
         astrCT(6, 0) = "ND"
         astrCT(7, 0) = "NE"
         astrCT(8, 0) = "NM"
         astrCT(9, 0) = "OK"
         astrCT(10, 0) = "SD"
         astrCT(11, 0) = "TX"
         astrCT(12, 0) = "WY"
         astrCT(13, 0) = "Totals"
      Else
         ReDim astrCT(13, CInt(strDataCols) + 1)
         astrCT(0, 0) = "Category"
         astrCT(1, 0) = "AMPL"
         astrCT(2, 0) = "CAMC"
         astrCT(3, 0) = "CASD"
         astrCT(4, 0) = "CDPL"
         astrCT(5, 0) = "DVDP"
         astrCT(6, 0) = "RCVA"
         astrCT(7, 0) = "RCVH"
         astrCT(8, 0) = "SPKA"
         astrCT(9, 0) = "SPKB"
         astrCT(10, 0) = "SPKF"
         astrCT(11, 0) = "SPKS"
         astrCT(12, 0) = "TVRC"
         astrCT(13, 0) = "Totals"
      End If

      'Generate the time-series column headers
      For intCol = 1 To CInt(strDataCols)
         Select Case strInterval
            Case "Quarter"
               datPeriod = datStartDate.AddMonths(3 * (intCol - 1))
               astrCT(0, intCol) = CStr(datPeriod.Year) + "Q" + CStr((datPeriod.Month \ 3) + 1)
            Case "Month"
               datPeriod = datStartDate.AddMonths(intCol - 1)
               astrCT(0, intCol) = Format(datPeriod, "MMM") + " " + CStr(datPeriod.Year)
            Case "Week"
               datPeriod = datStartDate.AddDays(7 * (intCol - 1))
               If datStartDate = #12/31/2000# And intCol = 1 Then
                  'Special case for 2001, which starts on Monday
                  astrCT(0, intCol) = "2001W1"
               Else
                  astrCT(0, intCol) = CStr(datPeriod.Year) + "W" + CStr((datPeriod.DayOfYear \ 7) + 1)
               End If
            Case "Day"
               datPeriod = datStartDate.AddDays(intCol - 1)
               astrCT(0, intCol) = Format(datPeriod, "M/d/yyyy")
         End Select
      Next intCol
      astrCT(0, CInt(strDataCols) + 1) = "Totals"

      'Fill array with 0s
      For intRow = 1 To UBound(astrCT, 1)
         For intCol = 1 To UBound(astrCT, 2)
            astrCT(intRow, intCol) = "0"
         Next intCol
      Next intRow

      'Connect and create a SqlDataReader
      Trace.Write("Begin connect to server")
      With cnnCT
         .ConnectionString = "Data Source=" + hdrAuthent.ServerName + ";Initial Catalog=OCE_Cust;UID=" + hdrAuthent.UserName + ";PWD=" + hdrAuthent.Password
         .Open()
         cmmCT = .CreateCommand
      End With
      With cmmCT
         .CommandType = CommandType.Text
         .CommandText = strSQL
         sdrCT = .ExecuteReader
      End With
      Trace.Write("Begin fill array with data")
      With sdrCT
         While .Read
            'Iterate the rowset and plug values into the proper cell
            For intRow = 1 To UBound(astrCT, 1)
               'Find the row
               If sdrCT.GetString(0) = astrCT(intRow, 0) Then
                  Exit For
               End If
            Next intRow
            For intCol = 1 To UBound(astrCT, 2)
               'Find the column
               If sdrCT.GetString(1) = astrCT(0, intCol) Then
                  Exit For
               End If
            Next intCol
            If intRow < UBound(astrCT, 1) And intCol < UBound(astrCT, 2) Then
               'Found the cell; add the value without decimals
               astrCT(intRow, intCol) = CStr(CInt(sdrCT.GetDecimal(3)))
            Else
               'Error below for test
               strTemp = sdrCT.GetString(0) + ", " + sdrCT.GetString(1) + ", " + CStr(sdrCT.GetDecimal(3))
            End If
         End While
      End With
      sdrCT.Close()
      cnnCT.Close()
      'Get row totals
      Trace.Write("Begin row/column totals")
      For intRow = 1 To UBound(astrCT, 1) - 1
         intTotal = 0
         For intCol = 1 To UBound(astrCT, 2) - 1
            intTotal += CInt(astrCT(intRow, intCol))
         Next intCol
         astrCT(intRow, UBound(astrCT, 2)) = CStr(intTotal)
      Next intRow
      'Get column totals (including crossfoot total)
      For intCol = 1 To UBound(astrCT, 2)
         intTotal = 0
         For intRow = 1 To UBound(astrCT, 1) - 1
            intTotal += CInt(astrCT(intRow, intCol))
         Next intRow
         astrCT(UBound(astrCT, 1), intCol) = CStr(intTotal)
      Next intCol
      'Format rows and columns
      For intRow = 1 To UBound(astrCT, 1)
         For intCol = 1 To UBound(astrCT, 2)
            If intRow = 1 Or intRow = UBound(astrCT, 1) Or intCol = UBound(astrCT, 2) Then
               astrCT(intRow, intCol) = Format(CInt(astrCT(intRow, intCol)), "$#,##0")
            Else
               astrCT(intRow, intCol) = Format(CInt(astrCT(intRow, intCol)), "#,##0")
            End If
         Next intCol
      Next intRow

      'Generate the response document
      Trace.Write("Begin generate response")
      strXML = "<crosstab xmlns='http://oakleaf.ws/services/crosstabs/response' >" + strCrLf
      For intRow = 0 To UBound(astrCT, 1)
         If intRow = UBound(astrCT, 1) And Not blnColTotals Then
            Exit For
         End If
         If intRow = 0 Then
            'Title
            strXML += "   <title>" + strTitle + "</title>" + strCrLf
            'Header
            strXML += "   <header>" + strCrLf + Space(6) + "<row>" + strCrLf
            For intCol = 0 To UBound(astrCT, 2)
               If intCol = UBound(astrCT, 2) And Not blnRowTotals Then
                  Exit For
               End If
               strAttrib = "id='hdr_col_" + astrCT(0, intCol) + "' cell='R1C" + CStr(intCol + 1) + "'"
               strXML += Space(9) + "<col " + strAttrib + ">" + astrCT(intRow, intCol) + "</col>" + strCrLf
            Next intCol
            strXML += Space(6) + "</row>" + strCrLf + "   </header>" + strCrLf
            strXML += "   <data>" + strCrLf
         Else
            'Data rows
            strXML += Space(6) + "<row>" + strCrLf
            For intCol = 0 To UBound(astrCT, 2)
               If intCol = UBound(astrCT, 2) And Not blnRowTotals Then
                  Exit For
               End If
               If intRow = UBound(astrCT, 1) Or intCol = UBound(astrCT, 2) Then
                  strAttrib = "id='total' cell='R" + CStr(intRow + 1) + "C" + CStr(intCol + 1) + "'"
               Else
                  If intCol > 0 Then
                     If blnStates Then
                        strAttrib = "id='st_"
                     Else
                        strAttrib = "id='cat_"
                     End If
                     strAttrib += astrCT(intRow, 0) + "_" + astrCT(0, intCol)
                  Else
                     strAttrib = "id='hdr_row_" + astrCT(intRow, 0)
                  End If
                  strAttrib += "' cell='R" + CStr(intRow + 1) + "C" + CStr(intCol + 1) + "'"
               End If
               If (astrCT(intRow, intCol) = "0" Or astrCT(intRow, intCol) = "$0") And Not blnNullToZero Then
                  'Empty element for zero values
                  strXML += Space(9) + "<col " + strAttrib + "></col>" + strCrLf
               Else
                  strXML += Space(9) + "<col " + strAttrib + ">" + astrCT(intRow, intCol) + "</col>" + strCrLf
               End If
            Next intCol
            strXML += Space(6) + "</row>" + strCrLf
         End If
      Next intRow
      'Close the tags
      strXML += "   </data>" + strCrLf
      strXML += "</crosstab>"
      Trace.Write("End generate response")
      Return strXML
   End Function
End Class
